/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.situation;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modelliert ein Objekt vom Datenverteiler-Typ typ.situation mit seinen
 * (parametrierenden) Daten der Attributgruppe atg.situationsEigenschaften.
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public abstract class Situation implements ClientReceiverInterface {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Systemobjekt vom Typ Situation.
	 */
	private SystemObject davObjekt;

	public SystemObject getDavObjekt() {
		return davObjekt;
	}

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavInterface connection;

	public ClientDavInterface getConnection() {
		return connection;
	}

	/**
	 * Datenbeschreibung für Datensatz der Attributgruppe
	 * atg.situationsEigenschaften.
	 */
	private final DataDescription situationsEigenschaftenBeschreibung;

	public DataDescription getSituationsEigenschaftenBeschreibung() {
		return situationsEigenschaftenBeschreibung;
	}

	/**
	 * Sammlung aller <code>SituationsEigenschaftenListener</code>.
	 */
	private Collection<SituationEigenschaftenListener> situationsEigenschaftenListener = new ArrayList<SituationEigenschaftenListener>();

	/**
	 * Konstruiert eine Instanz vom Typ Situation und führt die
	 * Empfangsanmeldung durch.
	 * 
	 * @param connection
	 *            Datenverteiler-Verbindung
	 * @param davObjekt
	 *            Systemobjekt vom Typ Situation
	 */
	protected Situation(final ClientDavInterface connection, final SystemObject davObjekt) {
		this.connection = connection;
		this.davObjekt = davObjekt;

		final AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.situationsEigenschaften");
		final Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");
		situationsEigenschaftenBeschreibung = new DataDescription(atg, asp);

		connection.subscribeReceiver(this, davObjekt, situationsEigenschaftenBeschreibung, ReceiveOptions.normal(),
				ReceiverRole.receiver());
		DEBUG.finer(situationsEigenschaftenBeschreibung + " zum Empfang angemeldet für " + davObjekt);
	}

	/**
	 * Gibt belegte Resourcen frei.
	 */
	void dispose() {
		connection.unsubscribeReceiver(this, davObjekt, situationsEigenschaftenBeschreibung);
		DEBUG.finer("Empfang abgemeldet von " + situationsEigenschaftenBeschreibung + " für " + davObjekt);
	}

	@Override
	public String toString() {
		return davObjekt != null ? davObjekt.getName() : "";
	}

	@Override
	public void update(final ResultData[] resultData) {
		for (ResultData data : resultData) {
			final Aspect asp = data.getDataDescription().getAspect();
			final AttributeGroup atg = data.getDataDescription().getAttributeGroup();

			if (atg.equals(situationsEigenschaftenBeschreibung.getAttributeGroup())
					&& asp.equals(situationsEigenschaftenBeschreibung.getAspect())) {
				DEBUG.fine("Update Daten (Parameter) " + atg.getPid() + " für " + this);

				uebernehmeDaten(data);
			}
		}
	}

	protected void uebernehmeDaten(final ResultData resultData) {
		setSituationEigenschaften(resultData.hasData() ? resultData.getData() : null);
	}

	public void addSituationsEigenschaftenListener(final SituationEigenschaftenListener listener) {
		synchronized (situationsEigenschaftenListener) {
			situationsEigenschaftenListener.add(listener);
		}
	}

	public void removeSituationsEigenschaftenListener(final SituationEigenschaftenListener listener) {
		synchronized (situationsEigenschaftenListener) {
			situationsEigenschaftenListener.remove(listener);
		}
	}

	protected void fireSituationsEigenschaftenListener(final SituationEigenschaften daten) {
		synchronized (situationsEigenschaftenListener) {
			for (SituationEigenschaftenListener listener : situationsEigenschaftenListener) {
				listener.neueSituationEigenschaften(this, daten);
			}
		}
	}

	/**
	 * Aktueller Datensatz der Attributgruppe atg.situationsEigenschaften für
	 * dieses Objekt.
	 */
	private Data situationEigenschaften;

	/**
	 * Zugriff auf die aktuellen Daten.
	 * 
	 * @return den aktueller Datensatz der Attributgruppe
	 *         atg.situationsEigenschaften für dieses Objekt
	 */
	protected Data getSituationsEigenschaften() {
		return situationEigenschaften;
	}

	protected void setSituationEigenschaften(final Data situationEigenschaften) {
		this.situationEigenschaften = situationEigenschaften;
	}
}
